@@ -2,8 +2,9 @@ import logging
 from gladier.client import GladierBaseClient
 from gladier.base import GladierBaseTool
 from gladier.decorators import generate_flow_definition
+from gladier.managers import FlowsManager
 
-__all__ = [GladierBaseClient, GladierBaseTool, generate_flow_definition]
+__all__ = [GladierBaseClient, GladierBaseTool, generate_flow_definition, FlowsManager]
 
 # https://docs.python.org/3/howto/logging.html#configuring-logging-for-a-library  # noqa
 logging.getLogger("gladier").addHandler(logging.NullHandler())
