@@ -1,6 +1,4 @@
 import logging
-import json
-from collections import OrderedDict
 from gladier.base import GladierBaseTool
 from gladier.client import GladierBaseClient
 from gladier.exc import FlowGenException
@@ -23,11 +21,8 @@ def combine_tool_flows(client: GladierBaseClient, modifiers):
     Modifiers can be applied to any of the states within the flow.
     """
     flow_moder = FlowModifiers(client.tools, modifiers, cls=client)
-    tool_chain = ToolChain(client.tools, flow_comment=client.__doc__)
-    tool_chain.compile_flow()
-
-    flow_def = flow_moder.apply_modifiers(tool_chain.flow_definition)
-    return json.loads(json.dumps(flow_def))
+    chain = ToolChain(flow_comment=client.__doc__).chain(client.tools)
+    return flow_moder.apply_modifiers(chain.flow_definition)
 
 
 def generate_tool_flow(tool: GladierBaseTool, modifiers):
@@ -36,37 +31,35 @@ def generate_tool_flow(tool: GladierBaseTool, modifiers):
 
     flow_moder = FlowModifiers([tool], modifiers, cls=tool)
 
-    flow_states = OrderedDict()
+    tools = ToolChain()
     for fx_func in tool.funcx_functions:
-        fx_state = generate_funcx_flow_state(fx_func)
-        flow_states.update(fx_state)
+        tools.chain_state(*generate_funcx_flow_state(fx_func))
 
-    if not flow_states:
+    flow = tools.flow_definition
+    if not flow['States']:
         raise FlowGenException(f'Tool {tool} has no flow states. Add a list of python functions '
                                f'as "{tool}.funcx_functions = [myfunction]" or set a custom flow '
                                f'definition instead using `{tool}.flow_definition = mydef`')
-    flow_def = ToolChain.combine_flow_states(flow_states, flow_comment=tool.__doc__)
-    flow_def = flow_moder.apply_modifiers(flow_def)
-    return json.loads(json.dumps(flow_def))
+    return flow_moder.apply_modifiers(flow)
 
 
 def generate_funcx_flow_state(funcx_function):
-
     state_name = get_funcx_flow_state_name(funcx_function)
-    tasks = [OrderedDict([
-        ('endpoint.$', '$.input.funcx_endpoint_compute'),
-        ('function.$', f'$.input.{get_funcx_function_name(funcx_function)}'),
-        ('payload.$', '$.input'),
-    ])]
-    flow_state = OrderedDict([
-        ('Comment', funcx_function.__doc__),
-        ('Type', 'Action'),
-        ('ActionUrl', 'https://automate.funcx.org'),
-        ('ActionScope', 'https://auth.globus.org/scopes/'
-                        'b3db7e59-a6f1-4947-95c2-59d6b7a70f8c/action_all'),
-        ('ExceptionOnActionFailure', False),
-        ('Parameters', OrderedDict(tasks=tasks)),
-        ('ResultPath', f'$.{state_name}'),
-        ('WaitTime', 300),
-    ])
-    return OrderedDict([(state_name, flow_state)])
+
+    return state_name, {
+        'Comment': funcx_function.__doc__,
+        'Type': 'Action',
+        'ActionUrl': 'https://automate.funcx.org',
+        'ActionScope': 'https://auth.globus.org/scopes/'
+                       'b3db7e59-a6f1-4947-95c2-59d6b7a70f8c/action_all',
+        'ExceptionOnActionFailure': False,
+        'Parameters': {
+            'tasks': [{
+                'endpoint.$': '$.input.funcx_endpoint_compute',
+                'function.$': f'$.input.{get_funcx_function_name(funcx_function)}',
+                'payload.$': '$.input',
+            }]
+        },
+        'ResultPath': f'$.{state_name}',
+        'WaitTime': 300,
+    }
